import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.StringTokenizer;

import javax.imageio.ImageIO;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.BytesWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Counter;
import org.apache.hadoop.mapreduce.Counters;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.FileSplit;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.NullOutputFormat;
import org.apache.hadoop.util.GenericOptionsParser;

public class SobelProcessing
{

	public static class SobelMapper extends Mapper<NullWritable, BytesWritable, NullWritable, NullWritable>
	{

		public void map(NullWritable key, BytesWritable value, Context context) throws IOException, InterruptedException
		{
			Configuration conf = context.getConfiguration();
			String filename = ((FileSplit) context.getInputSplit()).getPath().toString();
			String output_dir = conf.get("output.dir");
			filename = getFileName(filename);
			FileSystem fs = FileSystem.get(conf);
			FSDataOutputStream dos = fs.create(new Path(output_dir + filename+ ".jpg"));
			BufferedImage src = ImageIO.read(new ByteArrayInputStream(value.getBytes()));

			float sobscale = Float.valueOf(conf.get("sobscale"));
			int offsetval = Integer.valueOf(conf.get("offsetval"));

			int iw = src.getWidth();
			int ih = src.getHeight();
			BufferedImage dest = new BufferedImage(iw, ih, src.getType());

			int[][] gray = new int[iw][ih];

			for (int x = 0; x < iw; x++)
			{
				for (int y = 0; y < ih; y++)
				{
					int rgb = src.getRGB(x, y);
					int r = 0xFF & (rgb >> 16);
					int g = 0xFF & (rgb >> 8);
					int b = 0xFF & rgb;
					gray[x][y] = (int) (0.299 * r + 0.587 * g + 0.114 * b);
				}
			}

			for (int x = 1; x < iw - 1; x++)
			{
				for (int y = 1; y < ih - 1; y++)
				{
					int a = gray[x - 1][y - 1];
					int b = gray[x][y - 1];
					int c = gray[x + 1][y - 1];
					int d = gray[x - 1][y];
					int e = gray[x + 1][y];
					int f = gray[x - 1][y + 1];
					int g = gray[x][y + 1];
					int h = gray[x + 1][y + 1];

					int hor = (a + d + f) - (c + e + h);

					if (hor < 0)
						hor = -hor;

					int vert = (a + b + c) - (f + g + h);

					if (vert < 0)
						vert = -vert;

					int gc = (int) (sobscale * (hor + vert));
					gc = (gc + offsetval);

					if (gc > 255)
						gc = 255;

					int sobel = 0xff000000 | gc << 16 | gc << 8 | gc;
					dest.setRGB(x, y, sobel);
				}
			}

			ImageIO.write(dest, "jpg", dos);
			dos.close();

		}

		public String getFileName(String s)
		{
			return s.substring(s.lastIndexOf("/"), s.lastIndexOf("."));
		}

	}

	public static void main(String[] args) throws Exception
	{
		Configuration conf = new Configuration();
		conf.set("sobscale", "1.0");
		conf.set("offsetval", "0");
		conf.set("output.dir", args[1]);

		Job job = new Job(conf, "Sobel Proccessing");
		job.setJarByClass(SobelProcessing.class);
		job.setMapperClass(SobelMapper.class);
		job.setNumReduceTasks(0);
		job.setInputFormatClass(WholeFileInputFormat.class);
		job.setOutputFormatClass(NullOutputFormat.class);
		WholeFileInputFormat.addInputPath(job, new Path(args[0]));
		boolean status = job.waitForCompletion(true);
		System.exit(status ? 0 : 1);
	}
}
