import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.BytesWritable;
import org.apache.hadoop.io.IOUtils;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.FileSplit;

public class WholeFileRecordReader extends RecordReader<NullWritable, BytesWritable>
{
	private FileSplit fileSplit;
	private FSDataInputStream in;
	private BytesWritable value = new BytesWritable();

	public WholeFileRecordReader(FileSplit split, Configuration configuration)
			throws IOException
	{
		this.fileSplit = split;
		Path file = this.fileSplit.getPath();
		FileSystem fs = file.getFileSystem(configuration);
		this.in = fs.open(file);
	}

	public void close() throws IOException
	{
		IOUtils.closeStream(in);
	}

	public float getProgress()
	{
		try
		{
			return in.getPos() / fileSplit.getLength();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public NullWritable getCurrentKey() throws IOException, InterruptedException
	{
		return NullWritable.get();
	}

	@Override
	public BytesWritable getCurrentValue() throws IOException, InterruptedException
	{
		return value;
	}

	@Override
	public void initialize(InputSplit fileSplit, TaskAttemptContext arg1) throws IOException, InterruptedException
	{

	}

	@Override
	public boolean nextKeyValue() throws IOException, InterruptedException
	{
		if (in.getPos() == 0)
		{
			byte[] contents = new byte[(int) fileSplit.getLength()];
			IOUtils.readFully(in, contents, 0, contents.length);
			value.set(contents, 0, contents.length);
			return true;
		}
		return false;
	}

}
